from contextlib import ContextDecorator

__all_assertion_errors__ = []
__item_assertion_errors__ = []


class SoftAssertionError(AssertionError):
    pass


def clear_all_failures():
    """
    Очистить список всех ошибок
    """
    global __all_assertion_errors__
    __all_assertion_errors__ = []


def clear_item_failures():
    """
    Очистить список ошибок
    """
    global __item_assertion_errors__
    __item_assertion_errors__ = []


def merge_failures():
    global __all_assertion_errors__, __item_assertion_errors__
    __all_assertion_errors__.extend(__item_assertion_errors__)
    clear_item_failures()


def get_all_failures():
    """
    Получить список всех ошибок
    :return: список всех ошибок
    """
    return __all_assertion_errors__


class checker(ContextDecorator):
    """
    Класс мягких и мульти проверок
    """
    def __init__(self):
        merge_failures()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        errors = __item_assertion_errors__
        merge_failures()

        if exc_val and not isinstance(exc_val, SoftAssertionError):
            if not list(filter(lambda e: id(e) == id(exc_val), __all_assertion_errors__)):
                __all_assertion_errors__.append(exc_val)

        if errors and not exc_val:
            raise SoftAssertionError('\n'.join(errors))

    @staticmethod
    def wait(condition, message):
        """
        Метод отложенной проверки
        :param condition: условие, например: 1 > 2
        :param message: сообщение при невыполнении условия
        """
        if not condition:
            global __item_assertion_errors__
            __item_assertion_errors__.append(message)

    class soft(ContextDecorator):
        """
        Класс мягкой проверки. Гасит AssertionError если внутри декорируемой функции func сработал assertion
        """
        def __init__(self, *extra_exceptions):
            self.exceptions = extra_exceptions + (AssertionError, SoftAssertionError)

        def __enter__(self):
            return self

        def __exit__(self, exc_type, exc_val, exc_tb):
            global __all_assertion_errors__
            merge_failures()
            if exc_val and not isinstance(exc_val, SoftAssertionError):
                if not list(filter(lambda e: id(e) == id(exc_val), __all_assertion_errors__)):
                    __all_assertion_errors__.append(exc_val)
            if isinstance(exc_val, self.exceptions):
                return True


assertions = checker
