import pytest
from _pytest._code import ExceptionInfo

from . import check


@pytest.hookimpl(hookwrapper=True, trylast=True)
def pytest_runtest_makereport(item, call):
    outcome = yield

    check.merge_failures()
    failures = check.get_all_failures()
    check.clear_all_failures()

    if failures:
        failures = [str(e) for e in failures]
        if not call.excinfo:
            try:
                raise AssertionError('\n'.join(failures))
            except AssertionError:
                call.excinfo = ExceptionInfo.from_current()

        report = outcome.get_result()

        summary = "Failed Checks: {}".format(len(failures))
        longrepr = ["\n".join(failures), "-" * 60, summary]
        if report.longrepr:
            longrepr.append("-" * 60)
            longrepr.append(report.longreprtext)
            report.longrepr = "\n".join(longrepr)
        else:
            report.longrepr = "\n".join(longrepr)
        report.outcome = "failed"
